Set Warnings "-notation-overridden,-parsing".
From Coq Require Export String.
From VFA Require Import Redblack.

Parameter MISSING: Type.

Module Check.

Ltac check_type A B :=
    match type of A with
    | context[MISSING] => idtac "Missing:" A
    | ?T => first [unify T B; idtac "Type: ok" | idtac "Type: wrong - should be (" B ")"]
    end.

Ltac print_manual_grade A :=
    match eval compute in A with
    | Some (_ ?S ?C) =>
        idtac "Score:"  S;
        match eval compute in C with
          | ""%string => idtac "Comment: None"
          | _ => idtac "Comment:" C
        end
    | None =>
        idtac "Score: Ungraded";
        idtac "Comment: None"
    end.

End Check.

From VFA Require Import Redblack.
Import Check.

Goal True.

idtac "-------------------  empty_tree_BST  --------------------".
idtac " ".

idtac "#> empty_tree_BST".
idtac "Possible points: 1".
check_type @empty_tree_BST ((forall V : Type, BST V (empty_tree V))).
idtac "Assumptions:".
Abort.
Print Assumptions empty_tree_BST.
Goal True.
idtac " ".

idtac "-------------------  balanceP  --------------------".
idtac " ".

idtac "#> balanceP".
idtac "Possible points: 2".
check_type @balanceP (
(forall (V : Type) (P : Extract.int -> V -> Prop) 
   (l : tree V) (c : color) (r : tree V) (k : Extract.int) 
   (v : V),
 ForallT V P l -> ForallT V P r -> P k v -> ForallT V P (balance V c l k v r))).
idtac "Assumptions:".
Abort.
Print Assumptions balanceP.
Goal True.
idtac " ".

idtac "-------------------  insP  --------------------".
idtac " ".

idtac "#> insP".
idtac "Possible points: 2".
check_type @insP (
(forall (V : Type) (P : Extract.int -> V -> Prop) 
   (t : tree V) (k : Extract.int) (v : V),
 ForallT V P t -> P k v -> ForallT V P (ins V k v t))).
idtac "Assumptions:".
Abort.
Print Assumptions insP.
Goal True.
idtac " ".

idtac "-------------------  ins_BST  --------------------".
idtac " ".

idtac "#> ins_BST".
idtac "Possible points: 3".
check_type @ins_BST (
(forall (V : Type) (t : tree V) (k : key) (v : V),
 BST V t -> BST V (ins V k v t))).
idtac "Assumptions:".
Abort.
Print Assumptions ins_BST.
Goal True.
idtac " ".

idtac "-------------------  insert_BST  --------------------".
idtac " ".

idtac "#> insert_BST".
idtac "Possible points: 2".
check_type @insert_BST (
(forall (V : Type) (t : tree V) (v : V) (k : key),
 BST V t -> BST V (insert V k v t))).
idtac "Assumptions:".
Abort.
Print Assumptions insert_BST.
Goal True.
idtac " ".

idtac "-------------------  balance_lookup  --------------------".
idtac " ".

idtac "#> balance_lookup".
idtac "Possible points: 3".
check_type @balance_lookup (
(forall (V : Type) (default : V) (k : key) (c : color) 
   (k0 : Extract.int) (v0 : V) (l r : tree V),
 BST V l ->
 BST V r ->
 ForallT V
   (fun (k' : Extract.int) (_ : V) =>
    BinInt.Z.lt (Extract.int2Z k') (Extract.int2Z k0)) l ->
 ForallT V
   (fun (k' : Extract.int) (_ : V) =>
    BinInt.Z.gt (Extract.int2Z k') (Extract.int2Z k0)) r ->
 lookup V default k (balance V c l k0 v0 r) =
 (if BinInt.Z.ltb (Extract.int2Z k) (Extract.int2Z k0)
  then lookup V default k l
  else
   if BinInt.Z.ltb (Extract.int2Z k0) (Extract.int2Z k)
   then lookup V default k r
   else v0))).
idtac "Assumptions:".
Abort.
Print Assumptions balance_lookup.
Goal True.
idtac " ".

idtac "-------------------  lookup_ins_eq  --------------------".
idtac " ".

idtac "#> lookup_ins_eq".
idtac "Possible points: 2".
check_type @lookup_ins_eq (
(forall (V : Type) (default : V) (t : tree V) (k : key) (v : V),
 BST V t -> lookup V default k (ins V k v t) = v)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_ins_eq.
Goal True.
idtac " ".

idtac "-------------------  lookup_ins_neq  --------------------".
idtac " ".

idtac "#> lookup_ins_neq".
idtac "Possible points: 3".
check_type @lookup_ins_neq (
(forall (V : Type) (default : V) (t : tree V) (k : key) (v : V) (k0 : key),
 BST V t ->
 k <> k0 -> lookup V default k0 (ins V k v t) = lookup V default k0 t)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_ins_neq.
Goal True.
idtac " ".

idtac "-------------------  lookup_insert  --------------------".
idtac " ".

idtac "#> lookup_insert_eq".
idtac "Possible points: 2".
check_type @lookup_insert_eq (
(forall (V : Type) (default : V) (t : tree V) (k : key) (v : V),
 BST V t -> lookup V default k (insert V k v t) = v)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_insert_eq.
Goal True.
idtac " ".

idtac "#> lookup_insert_neq".
idtac "Possible points: 1".
check_type @lookup_insert_neq (
(forall (V : Type) (default : V) (t : tree V) (k : key) (v : V) (k0 : key),
 BST V t ->
 k <> k0 -> lookup V default k0 (insert V k v t) = lookup V default k0 t)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_insert_neq.
Goal True.
idtac " ".

idtac "-------------------  is_redblack_properties  --------------------".
idtac " ".

idtac "#> is_redblack_toblack".
idtac "Possible points: 1.5".
check_type @is_redblack_toblack (
(forall (V : Type) (s : tree V) (n : nat),
 is_redblack V s Red n -> is_redblack V s Black n)).
idtac "Assumptions:".
Abort.
Print Assumptions is_redblack_toblack.
Goal True.
idtac " ".

idtac "#> makeblack_fiddle".
idtac "Possible points: 1.5".
check_type @makeblack_fiddle (
(forall (V : Type) (s : tree V) (n : nat),
 is_redblack V s Black n ->
 exists n0 : nat, is_redblack V (makeBlack V s) Red n0)).
idtac "Assumptions:".
Abort.
Print Assumptions makeblack_fiddle.
Goal True.
idtac " ".

idtac "#> ins_is_redblack".
idtac "Possible points: 1.5".
check_type @ins_is_redblack (
(forall (V : Type) (x : key) (vx : V) (s : tree V) (n : nat),
 (is_redblack V s Black n -> nearly_redblack V (ins V x vx s) n) /\
 (is_redblack V s Red n -> is_redblack V (ins V x vx s) Black n))).
idtac "Assumptions:".
Abort.
Print Assumptions ins_is_redblack.
Goal True.
idtac " ".

idtac "#> insert_is_redblack".
idtac "Possible points: 1.5".
check_type @insert_is_redblack (
(forall (V : Type) (x : key) (xv : V) (s : tree V) (n : nat),
 is_redblack V s Red n ->
 exists n' : nat, is_redblack V (insert V x xv s) Red n')).
idtac "Assumptions:".
Abort.
Print Assumptions insert_is_redblack.
Goal True.
idtac " ".

idtac " ".

idtac "Max points - standard: 27".
idtac "Max points - advanced: 27".
idtac "".
idtac "Allowed Axioms:".
idtac "functional_extensionality".
idtac "functional_extensionality_dep".
idtac "FunctionalExtensionality.functional_extensionality_dep".
idtac "int".
idtac "int2Z".
idtac "int2Z_inj".
idtac "ltb_lt".
idtac "ltb".
idtac "Extract.int".
idtac "Extract.int2Z".
idtac "Extract.int2Z_inj".
idtac "Extract.ltb_lt".
idtac "Extract.ltb".
idtac "".
idtac "".
idtac "********** Summary **********".
idtac "".
idtac "Below is a summary of the automatically graded exercises that are incomplete.".
idtac "".
idtac "The output for each exercise can be any of the following:".
idtac "  - 'Closed under the global context', if it is complete".
idtac "  - 'MANUAL', if it is manually graded".
idtac "  - A list of pending axioms, containing unproven assumptions. In this case".
idtac "    the exercise is considered complete, if the axioms are all allowed.".
idtac "".
idtac "********** Standard **********".
idtac "---------- empty_tree_BST ---------".
Print Assumptions empty_tree_BST.
idtac "---------- balanceP ---------".
Print Assumptions balanceP.
idtac "---------- insP ---------".
Print Assumptions insP.
idtac "---------- ins_BST ---------".
Print Assumptions ins_BST.
idtac "---------- insert_BST ---------".
Print Assumptions insert_BST.
idtac "---------- balance_lookup ---------".
Print Assumptions balance_lookup.
idtac "---------- lookup_ins_eq ---------".
Print Assumptions lookup_ins_eq.
idtac "---------- lookup_ins_neq ---------".
Print Assumptions lookup_ins_neq.
idtac "---------- lookup_insert_eq ---------".
Print Assumptions lookup_insert_eq.
idtac "---------- lookup_insert_neq ---------".
Print Assumptions lookup_insert_neq.
idtac "---------- is_redblack_toblack ---------".
Print Assumptions is_redblack_toblack.
idtac "---------- makeblack_fiddle ---------".
Print Assumptions makeblack_fiddle.
idtac "---------- ins_is_redblack ---------".
Print Assumptions ins_is_redblack.
idtac "---------- insert_is_redblack ---------".
Print Assumptions insert_is_redblack.
idtac "".
idtac "********** Advanced **********".
Abort.

(* Tue Jan 28 15:14:06 EST 2020 *)

(* Tue Jan 28 15:14:06 EST 2020 *)
