Set Warnings "-notation-overridden,-parsing".
From Coq Require Export String.
From VFA Require Import SearchTree.

Parameter MISSING: Type.

Module Check.

Ltac check_type A B :=
    match type of A with
    | context[MISSING] => idtac "Missing:" A
    | ?T => first [unify T B; idtac "Type: ok" | idtac "Type: wrong - should be (" B ")"]
    end.

Ltac print_manual_grade A :=
    match eval compute in A with
    | Some (_ ?S ?C) =>
        idtac "Score:"  S;
        match eval compute in C with
          | ""%string => idtac "Comment: None"
          | _ => idtac "Comment:" C
        end
    | None =>
        idtac "Score: Ungraded";
        idtac "Comment: None"
    end.

End Check.

From VFA Require Import SearchTree.
Import Check.

Goal True.

idtac "-------------------  lookup_insert_neq  --------------------".
idtac " ".

idtac "#> lookup_insert_neq".
idtac "Possible points: 2".
check_type @lookup_insert_neq (
(forall (V : Type) (d : V) (t : tree V) (k : key) (v : V) (k' : key),
 k <> k' -> lookup V d k' (insert V k v t) = lookup V d k' t)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_insert_neq.
Goal True.
idtac " ".

idtac "-------------------  lookup_insert_shadow  --------------------".
idtac " ".

idtac "#> lookup_insert_shadow".
idtac "Possible points: 2".
check_type @lookup_insert_shadow (
(forall (V : Type) (d : V) (t : tree V) (k' k : key) (v v' : V),
 lookup V d k' (insert V k v (insert V k v' t)) =
 lookup V d k' (insert V k v t))).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_insert_shadow.
Goal True.
idtac " ".

idtac "-------------------  lookup_insert_same  --------------------".
idtac " ".

idtac "#> lookup_insert_same".
idtac "Possible points: 2".
check_type @lookup_insert_same (
(forall (V : Type) (d : V) (t : tree V) (k' k : key),
 lookup V d k' (insert V k (lookup V d k t) t) = lookup V d k' t)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_insert_same.
Goal True.
idtac " ".

idtac "-------------------  lookup_insert_permute  --------------------".
idtac " ".

idtac "#> lookup_insert_permute".
idtac "Possible points: 2".
check_type @lookup_insert_permute (
(forall (V : Type) (d : V) (t : tree V) (k1 k2 k' : key) (v1 v2 : V),
 k1 <> k2 ->
 lookup V d k' (insert V k1 v1 (insert V k2 v2 t)) =
 lookup V d k' (insert V k2 v2 (insert V k1 v1 t)))).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_insert_permute.
Goal True.
idtac " ".

idtac "-------------------  is_bst  --------------------".
idtac " ".

idtac "#> is_bst".
idtac "Possible points: 1".
check_type @is_bst (
(forall (V : Type) (v2 v4 v5 : V),
 BST V (T V (T V (E V) 2 v2 (E V)) 4 v4 (T V (E V) 5 v5 (E V))))).
idtac "Assumptions:".
Abort.
Print Assumptions is_bst.
Goal True.
idtac " ".

idtac "-------------------  is_not_bst  --------------------".
idtac " ".

idtac "#> is_not_bst".
idtac "Possible points: 1".
check_type @is_not_bst (
(forall (V : Type) (v2 v4 v5 : V),
 ~ BST V (T V (T V (E V) 4 v4 (E V)) 2 v2 (T V (E V) 5 v5 (E V))))).
idtac "Assumptions:".
Abort.
Print Assumptions is_not_bst.
Goal True.
idtac " ".

idtac "-------------------  empty_tree_BST  --------------------".
idtac " ".

idtac "#> empty_tree_BST".
idtac "Possible points: 1".
check_type @empty_tree_BST ((forall V : Type, BST V (empty_tree V))).
idtac "Assumptions:".
Abort.
Print Assumptions empty_tree_BST.
Goal True.
idtac " ".

idtac "-------------------  insert_BST  --------------------".
idtac " ".

idtac "#> insert_BST".
idtac "Possible points: 3".
check_type @insert_BST (
(forall (V : Type) (k : key) (v : V) (t : tree V),
 BST V t -> BST V (insert V k v t))).
idtac "Assumptions:".
Abort.
Print Assumptions insert_BST.
Goal True.
idtac " ".

idtac "-------------------  In_elements_lookup  --------------------".
idtac " ".

idtac "#> In_elements_lookup".
idtac "Possible points: 3".
check_type @In_elements_lookup (
(forall (V : Type) (d : V) (t : tree V),
 BST V t ->
 forall (k : key) (v : V),
 @List.In (key * V) (k, v) (elements V t) -> lookup V d k t = v)).
idtac "Assumptions:".
Abort.
Print Assumptions In_elements_lookup.
Goal True.
idtac " ".

idtac "-------------------  list2map_yes  --------------------".
idtac " ".

idtac "#> list2map_yes".
idtac "Possible points: 2".
check_type @list2map_yes (
(forall (V : Type) (el : list (key * V)),
 @List.NoDup key (@List.map (key * V) key (@fst key V) el) ->
 forall (k : key) (v : V),
 @List.In (key * V) (k, v) el -> list2map V el k = @Some V v)).
idtac "Assumptions:".
Abort.
Print Assumptions list2map_yes.
Goal True.
idtac " ".

idtac "-------------------  list2map_no  --------------------".
idtac " ".

idtac "#> list2map_no".
idtac "Possible points: 2".
check_type @list2map_no (
(forall (V : Type) (el : list (key * V)) (k : key),
 ~ @List.In key k (@List.map (key * V) key (@fst key V) el) ->
 list2map V el k = @None V)).
idtac "Assumptions:".
Abort.
Print Assumptions list2map_no.
Goal True.
idtac " ".

idtac "-------------------  lookup_relate  --------------------".
idtac " ".

idtac "#> lookup_relate".
idtac "Possible points: 3".
check_type @lookup_relate (
(forall (V : Type) (d : V) (t : tree V) (k : key),
 BST V t -> lookup V d k t = find V d (Abs V t) k)).
idtac "Assumptions:".
Abort.
Print Assumptions lookup_relate.
Goal True.
idtac " ".

idtac "-------------------  insert_relate  --------------------".
idtac " ".

idtac "#> insert_relate".
idtac "Possible points: 3".
check_type @insert_relate (
(forall (V : Type) (t : tree V) (k : key) (v : V),
 BST V t -> Abs V (insert V k v t) = @Maps.update V (Abs V t) k v)).
idtac "Assumptions:".
Abort.
Print Assumptions insert_relate.
Goal True.
idtac " ".

idtac " ".

idtac "Max points - standard: 27".
idtac "Max points - advanced: 27".
idtac "".
idtac "Allowed Axioms:".
idtac "functional_extensionality".
idtac "functional_extensionality_dep".
idtac "FunctionalExtensionality.functional_extensionality_dep".
idtac "int".
idtac "int2Z".
idtac "int2Z_inj".
idtac "ltb_lt".
idtac "ltb".
idtac "Extract.int".
idtac "Extract.int2Z".
idtac "Extract.int2Z_inj".
idtac "Extract.ltb_lt".
idtac "Extract.ltb".
idtac "".
idtac "".
idtac "********** Summary **********".
idtac "".
idtac "Below is a summary of the automatically graded exercises that are incomplete.".
idtac "".
idtac "The output for each exercise can be any of the following:".
idtac "  - 'Closed under the global context', if it is complete".
idtac "  - 'MANUAL', if it is manually graded".
idtac "  - A list of pending axioms, containing unproven assumptions. In this case".
idtac "    the exercise is considered complete, if the axioms are all allowed.".
idtac "".
idtac "********** Standard **********".
idtac "---------- lookup_insert_neq ---------".
Print Assumptions lookup_insert_neq.
idtac "---------- lookup_insert_shadow ---------".
Print Assumptions lookup_insert_shadow.
idtac "---------- lookup_insert_same ---------".
Print Assumptions lookup_insert_same.
idtac "---------- lookup_insert_permute ---------".
Print Assumptions lookup_insert_permute.
idtac "---------- is_bst ---------".
Print Assumptions is_bst.
idtac "---------- is_not_bst ---------".
Print Assumptions is_not_bst.
idtac "---------- empty_tree_BST ---------".
Print Assumptions empty_tree_BST.
idtac "---------- insert_BST ---------".
Print Assumptions insert_BST.
idtac "---------- In_elements_lookup ---------".
Print Assumptions In_elements_lookup.
idtac "---------- list2map_yes ---------".
Print Assumptions list2map_yes.
idtac "---------- list2map_no ---------".
Print Assumptions list2map_no.
idtac "---------- lookup_relate ---------".
Print Assumptions lookup_relate.
idtac "---------- insert_relate ---------".
Print Assumptions insert_relate.
idtac "".
idtac "********** Advanced **********".
Abort.

(* Tue Jan 28 15:14:03 EST 2020 *)

(* Tue Jan 28 15:14:03 EST 2020 *)
